﻿Imports System.ComponentModel

Public Class Form1
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        LoadDisable = True
        ' Populate Hashtable variables for various spells, talents, etc.
        Other.populateVariables()
        ' Make sure data directories exist.
        DataFiles.CheckDirs()
        ' Load Armor, Weapon, Enchant, and Gem data.
        DataFiles.NoneOption()
        DataFiles.loadGearData()
        DataFiles.loadEnchantData()
        DataFiles.loadGemData()
        DataFiles.loadSocketBonusData()
        ZoneSelect.SelectedItem = "US"
        LoadDisable = False
    End Sub

    Private Sub FireButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FireButton.Click
        If (checkValues() And sanityCheck()) Then
            FireButton.Enabled = False
            SSLabel.Text = "Executing run..."
            Me.Cursor = Cursors.WaitCursor
            Armory.processGearInfo()
            Me.updateRatings()
            preRunTasks()
            ' Spawn background thread.
            BGWork.RunWorkerAsync()
            ' If Model used, allow TenBox updates.
            If doModel.Checked = True Then
                TenBoxUpdate.Enabled = True
            Else
                TenBoxUpdate.Enabled = False
            End If
        End If
    End Sub

    Private Sub ClearButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ClearButton.Click
        TextBox1.Clear()
    End Sub

    Private Sub ImportButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ImportButton.Click
        If CharSelect.Text <> "" And RealmSelect.Text <> "" Then
            Variables.charName = CharSelect.Text
            Variables.charRealm = RealmSelect.Text
            Variables.charZone = ZoneSelect.Text
            SSLabel.Text = "Importing from Armory.  Please wait."
            ArmoryWorker.RunWorkerAsync()
        End If
    End Sub

    Private Sub TenBoxUpdate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TenBoxUpdate.Click
        If (checkValues() And sanityCheck()) Then
            TenBoxUpdate.Enabled = False
            preRunTasks()
            ' Spawn background thread.
            TenBoxWorker.RunWorkerAsync()
        End If
    End Sub

    '''''''''''''''''''''''''''''''''
    ' START WORKER THREAD FUNCTIONS
    Private Sub BGWork_DoWork(ByVal sender As Object, ByVal e As DoWorkEventArgs) Handles BGWork.DoWork
        Dim worker As BackgroundWorker = CType(sender, BackgroundWorker)
        If doModel.Checked Then
            Other.doModelRun(worker)
        Else
            Other.doSimulationRun(worker)
        End If
        e.Result = updateReceived(worker, e)
    End Sub

    Private Function updateReceived(ByVal worker As BackgroundWorker, ByVal e As DoWorkEventArgs) As String
        ' Accept updates from DoWork thread.  
        Dim result As String = ""
        If worker.CancellationPending Then
            worker.ReportProgress(100)
            e.Cancel = True
        Else
            worker.ReportProgress(100)
        End If
        Return result
    End Function

    Private Sub BGWork_Progress(ByVal sender As Object, ByVal e As ProgressChangedEventArgs) Handles BGWork.ProgressChanged
        Dim ustate As String = ""
        If e.UserState IsNot Nothing Then ustate = e.UserState.ToString
        SSLabel.Text = "Current time: " & ustate
    End Sub

    Private Sub BGWork_RunWorkerCompleted(ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs) Handles BGWork.RunWorkerCompleted
        If (e.Error IsNot Nothing) Then
            TextBox1.AppendText("There was an error in the operation:" & vbCrLf & e.Error.ToString)
        ElseIf e.Cancelled Then
            TextBox1.AppendText("Operation canceled.")
        Else
            SSLabel.Text = "Parsing damage results..."
            Other.parseDamageResults()
            TextBox1.AppendText(Spells.doLightningBoltStat())
            TextBox1.AppendText(Spells.doLavaBurstStat())
            TextBox1.AppendText(vbCrLf)
            updateCompareBoxes()
            Me.Cursor = Cursors.Arrow
            FireButton.Enabled = True
            SSLabel.Text = "Done."
        End If
    End Sub

    Private Sub ArmoryWorker_DoWork(ByVal sender As Object, ByVal e As DoWorkEventArgs) Handles ArmoryWorker.DoWork
        Dim worker As BackgroundWorker = CType(sender, BackgroundWorker)
        Armory.doArmoryImport(Variables.charName, Variables.charRealm, Variables.charZone, worker)
        e.Result = ArmoryUpdate(worker, e)
    End Sub

    Private Function ArmoryUpdate(ByVal worker As BackgroundWorker, ByVal e As DoWorkEventArgs) As String
        Dim result As String = ""
        If worker.CancellationPending Then
            e.Cancel = True
            result = "Import canceled."
        Else
            result = "Import finished."
        End If
        Return result
    End Function

    Private Sub Armory_Progress(ByVal sender As Object, ByVal e As ProgressChangedEventArgs) Handles ArmoryWorker.ProgressChanged
        Dim ustate As String = ""
        If e.UserState IsNot Nothing Then ustate = e.UserState.ToString
        SSLabel.Text = ustate
    End Sub

    Private Sub Armory_RunWorkerCompleted(ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs) Handles ArmoryWorker.RunWorkerCompleted
        Try
            If (e.Error IsNot Nothing) Then
                MessageBox.Show("There was an error importing Armory data.")
                TextBox1.AppendText(e.Error.ToString & vbCrLf)
            ElseIf e.Cancelled Then
                MessageBox.Show("Armory import canceled.")
            Else
                RaceSelect.Text = StoredRace
                SSLabel.Text = "Parsing talents..."
                Armory.parseTalents()
                SSLabel.Text = "Parsing gear..."
                Armory.parseGear()
                SSLabel.Text = "Processing gear..."
                Armory.processGearInfo()
                SSLabel.Text = "Updating ratings..."
                Me.updateRatings()
                Me.updateGemBoxes()
                SSLabel.Text = "Finished import."
                MessageBox.Show("Import complete.")
            End If
        Catch ex As Exception
            MessageBox.Show("Armory import error: " & ex.Message)
        End Try
    End Sub

    Private Sub TenBoxWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles TenBoxWorker.DoWork
        Dim worker As BackgroundWorker = CType(sender, BackgroundWorker)
        Dim newdps As Integer = 0
        ' Update SP Box
        Variables.StoredSpellDmg += 20
        Other.doModelRun()
        newdps = Other.parseTenResults()
        Variables.rawDamage.Clear()
        Variables.StoredTenSPBox = CInt(newdps) - CInt(DPSforCycle.Text)
        Variables.StoredSpellDmg -= 20
        ' Update Hit Box
        Variables.StoredHitRate += 20
        Other.doModelRun()
        newdps = Other.parseTenResults()
        Variables.rawDamage.Clear()
        Variables.StoredTenHitBox = CInt(newdps) - CInt(DPSforCycle.Text)
        Variables.StoredHitRate -= 20
        ' Update Haste Box
        Variables.StoredHasteRate += 20
        Other.doModelRun()
        newdps = Other.parseTenResults()
        Variables.rawDamage.Clear()
        Variables.StoredTenHasteBox = CInt(newdps) - CInt(DPSforCycle.Text)
        Variables.StoredHasteRate -= 20
        ' Update Crit Box
        Dim critinfo() As String = Split(Variables.Ratings.Item("Crit85"), ",")
        Dim addcrit As Decimal = 20 / CDec(critinfo(0))
        Variables.StoredCritPerc += addcrit
        Other.doModelRun()
        newdps = Other.parseTenResults()
        Variables.rawDamage.Clear()
        Variables.StoredTenCritBox = CInt(newdps) - CInt(DPSforCycle.Text)
        Variables.StoredCritPerc -= addcrit
        ' Update Mastery Box
        Variables.StoredEleOverChance = 16 + (((Variables.StoredMastRate + 20) / CDec(Variables.Ratings.Item("Mastery85"))) * 2)
        Other.doModelRun()
        newdps = Other.parseTenResults()
        Variables.rawDamage.Clear()
        Variables.StoredTenMasteryBox = CInt(newdps) - CInt(DPSforCycle.Text)
        Variables.StoredEleOverChance = 16 + (((Variables.StoredMastRate + 20) / CDec(Variables.Ratings.Item("Mastery85"))) * 2)

        e.Result = updateReceived(worker, e)
    End Sub

    Private Sub TenBox_RunWorkerCompleted(ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs) Handles TenBoxWorker.RunWorkerCompleted
        TenSPBox.Text = Variables.StoredTenSPBox.ToString
        TenHitBox.Text = Variables.StoredTenHitBox.ToString
        TenCritBox.Text = Variables.StoredTenCritBox.ToString
        TenHasteBox.Text = Variables.StoredTenHasteBox.ToString
        TenMasteryBox.Text = Variables.StoredTenMasteryBox.ToString
        TenBoxUpdate.Enabled = True
    End Sub
    ' END WORKER THREAD FUNCTIONS
    '''''''''''''''''''''''''''''''

    '''''''''''''''''''''
    ' START UI FUNCTIONS
    Private Sub updateGemBoxes()
        Dim hi() As String = Split(Variables.CurrentGear.Item("Head"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then HeadGemButt.Show() Else HeadGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Neck"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then NeckGemButt.Show() Else NeckGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Shoulder"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then ShoulderGemButt.Show() Else ShoulderGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Back"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then BackGemButt.Show() Else BackGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Chest"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then ChestGemButt.Show() Else ChestGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Wrist"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then WristGemButt.Show() Else WristGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("MainHand"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then MainHandGemButt.Show() Else MainHandGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("OffHand"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then OffhandGemButt.Show() Else OffhandGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Relic"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then RelicGemButt.Show() Else RelicGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Hands"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then HandsGemButt.Show() Else HandsGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Waist"), ";")
        If UBound(hi) > 0 Then
            ' Also, check to see if they have an Eternal Belt Buckle
            If (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Or Variables.CurrentEnchants.Item("Waist") = "1230012" Then WaistGemButt.Show() Else WaistGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Legs"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then LegsGemButt.Show() Else LegsGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Feet"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then FeetGemButt.Show() Else FeetGemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Finger1"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then Finger1GemButt.Show() Else Finger1GemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Finger2"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then Finger2GemButt.Show() Else Finger2GemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Trinket1"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then Trinket1GemButt.Show() Else Trinket1GemButt.Hide()
        End If
        hi = Split(Variables.CurrentGear.Item("Trinket2"), ";")
        If UBound(hi) > 0 Then
            If hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "" Then Trinket2GemButt.Show() Else Trinket2GemButt.Hide()
        End If
    End Sub

    Public Sub updateRatings()
        If RaceSelect.Text <> "" Then
            Dim addSpell As Integer = 0
            Dim addHit As Integer = 0
            Dim addHaste As Integer = 0
            Dim addCritP As Decimal = 0.0
            Dim addCritR As Integer = 0
            Dim addMastR As Integer = 0
            If AddSpellDmg.Text <> "" Then addSpell = CDbl(AddSpellDmg.Text)
            If AddHitRate.Text <> "" Then addHit = CDbl(AddHitRate.Text)
            If AddHasteRate.Text <> "" Then addHaste = CDbl(AddHasteRate.Text)
            If AddCritPerc.Text <> "" Then addCritP = CDec(AddCritPerc.Text) + 5 + CInt(Me.Talent_Acuity.Text) ' (5% from Elemental Oath, X% from Acuity)
            If AddCritRate.Text <> "" Then addCritR = CDbl(AddCritRate.Text)
            If AddMastRate.Text <> "" Then addMastR = CDbl(AddMastRate.Text)

            Dim ftcrit As Integer = 0
            Dim ftspell As Integer = 0
            If FlameTongueWeapon.Checked = True Then
                ftspell = System.Math.Round(Variables.Spells.Item("FLT85") * (1 + (Me.Talent_ElementalWeapons.Text / 5)), 0)
                If GoFTWCheckBox.Checked Then ftcrit = 2
            End If

            Dim totemSP As Decimal = 1.0
            If TotemicWrath.Checked Then totemSP = 1.1

            If Me.Talent_ElementalPrecision.Text <> "0" Then
                If Me.Talent_ElementalPrecision.Text = "1" Then addHit += System.Math.Round(CDec(Variables.SpiFromGear) * 0.33)
                If Me.Talent_ElementalPrecision.Text = "2" Then addHit += System.Math.Round(CDec(Variables.SpiFromGear) * 0.66)
                If Me.Talent_ElementalPrecision.Text = "3" Then addHit += CDec(Variables.SpiFromGear)
            End If

            Dim totalInt2SP As Integer = Variables.IntFromGear + (Variables.Intellect.Item(RaceSelect.Text) - 10)   ' First 10 points of Int are not counted
            If MailSpecialization.Checked Then totalInt2SP = Math.Floor(totalInt2SP * 1.05)

            SpellPower.Text = System.Math.Round((ftspell + addSpell + Variables.SPFromGear + totalInt2SP) * totemSP)
            HitRating.Text = Variables.HitFromGear + addHit
            HasteRating.Text = Variables.HasteFromGear + addHaste
            MasteryRating.Text = Variables.MastFromGear + addMastR
            Dim critinfo() As String = Split(Variables.Ratings.Item("Crit85"), ",")
            Dim baseint As Integer = Variables.Intellect.Item(RaceSelect.Text) - 10
            Dim cfg As Decimal = (Variables.CritFromGear + addCritR) / critinfo(0)
            Dim cfi As Decimal = ((Variables.IntFromGear + baseint) * critinfo(1)) + 2.2
            CritPercent.Text = System.Math.Round(cfg + cfi, 2) + ftcrit + addCritP
        End If
    End Sub

    Private Sub updateCompareBoxes()
        If Variables.PrevDPS <> 0 Then
            DiffDPSText.Text = CInt(DPSforCycle.Text) - Variables.PrevDPS
            DiffCritText.Text = CDec(EffectiveCritRate.Text.Replace("%", "")) - Variables.PrevCrit
            DiffHitText.Text = CDec(EffectiveHitRate.Text.Replace("%", "")) - Variables.PrevHit
            DiffOverText.Text = CDec(EffectiveMasteryRate.Text.Replace("%", "")) - Variables.PrevMast
        End If
        Variables.PrevDPS = DPSforCycle.Text

        Variables.PrevCrit = EffectiveCritRate.Text.Replace("%", "")
        Variables.PrevHit = EffectiveHitRate.Text.Replace("%", "")
        Variables.PrevMast = EffectiveMasteryRate.Text.Replace("%", "")
    End Sub

    Private Sub TextboxTalentChanges(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
    AddSpellDmg.TextChanged, AddHitRate.TextChanged, AddCritRate.TextChanged, AddCritPerc.TextChanged, AddHasteRate.TextChanged, AddMastRate.TextChanged, FlameTongueWeapon.CheckedChanged, _
    GoFTWCheckBox.CheckedChanged, MailSpecialization.CheckedChanged, TotemicWrath.CheckedChanged
        If LoadDisable = False Then Me.updateRatings()
    End Sub

    Private Sub GoLCheckBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GoLCheckBox.CheckedChanged
        If GoLCheckBox.Checked Then
            Variables.StoredGoLvBCheck = 0.1
        Else
            Variables.StoredGoLvBCheck = 0.0
        End If
    End Sub

    Private Sub GoLBCheckBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GoLBCheckBox.CheckedChanged
        If GoLBCheckBox.Checked Then
            Variables.StoredGoLBCheck = 0.04
        Else
            Variables.StoredGoLBCheck = 0
        End If
    End Sub

    Private Sub GoSCheckBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GoSCheckBox.CheckedChanged
        If GoSCheckBox.Checked Then
            Variables.StoredGoSCheck = 1
        Else
            Variables.StoredGoSCheck = 0
        End If
    End Sub

    Private Sub GoFSCheckBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GoFSCheckBox.CheckedChanged
        If GoFSCheckBox.Checked Then
            Variables.StoredFSDoTTime = 27
        Else
            Variables.StoredFSDoTTime = 18
        End If
    End Sub

    Public Sub sortGearDropDown()
        HeadCombo.Sorted = True
        NeckCombo.Sorted = True
        ShouldersCombo.Sorted = True
        BackCombo.Sorted = True
        ChestCombo.Sorted = True
        WristCombo.Sorted = True
        MainHandCombo.Sorted = True
        OffHandCombo.Sorted = True
        RelicCombo.Sorted = True
        HandsCombo.Sorted = True
        WaistCombo.Sorted = True
        LegsCombo.Sorted = True
        FeetCombo.Sorted = True
        Finger1Combo.Sorted = True
        Finger2Combo.Sorted = True
        Trinket1Combo.Sorted = True
        Trinket2Combo.Sorted = True
    End Sub
    ' END UI FUNCTIONS
    '''''''''''''''''''

    ''''''''''''''''''''''''''''
    ' START TOOLSTRIP FUNCTIONS
    Private Sub LoadSavedDataToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LoadSavedDataToolStripMenuItem.Click
        DataFiles.LoadCharacterData()
        ' Process Stats from Current Gear
        Armory.processGearInfo()
        ' Update Textboxes
        updateRatings()
        updateGemBoxes()
    End Sub

    Private Sub SaveCharacterDataToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveCharacterDataToolStripMenuItem.Click
        If CharSelect.Text <> "" And RealmSelect.Text <> "" Then
            DataFiles.SaveCharacterData()
            MessageBox.Show("Character Saved.")
        Else
            MessageBox.Show("No character selected.")
        End If
    End Sub

    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        Me.Close()
        Application.Exit()
    End Sub

    Private Sub GearToolStripMenuItem1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GearToolStripMenuItem1.Click
        Dim F2 As New ItemImporter_Gear
        F2.ShowDialog(Me)
    End Sub

    Private Sub GemToolStripMenuItem1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GemToolStripMenuItem1.Click
        Dim F2 As New ItemImporter_Gem
        F2.ShowDialog(Me)
    End Sub

    Private Sub EnchantToolStripMenuItem1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Dim F2 As New ItemImporter_Gear
        F2.ShowDialog(Me)
    End Sub

    Private Sub GearToolStripMenuItem2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GearToolStripMenuItem2.Click
        Dim F2 As New ItemEditor_Gear
        F2.ShowDialog(Me)
    End Sub

    Private Sub GemsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GemsToolStripMenuItem.Click
        Dim F2 As New ItemEditor_Gems
        F2.ShowDialog(Me)
    End Sub

    Private Sub EnchantsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EnchantsToolStripMenuItem.Click
        Dim F2 As New ItemEditor_Enchants
        F2.ShowDialog(Me)
    End Sub

    Private Sub SocketBonusToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SocketBonusToolStripMenuItem.Click
        Dim F2 As New ItemEditor_Sockets
        F2.ShowDialog(Me)
    End Sub

    Private Sub ComparisonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComparisonToolStripMenuItem.Click
        If (checkValues() And sanityCheck()) Then
            TenBoxUpdate.Enabled = False
            ' Move Default Gems into storage.
            Dim bg() As String = Split(Variables.GemHash_Name.Item(DefBlueGem.Text), ";")
            Variables.StoredDefBlueGem = bg(0)
            Dim rg() As String = Split(Variables.GemHash_Name.Item(DefRedGem.Text), ";")
            Variables.StoredDefRedGem = rg(0)
            Dim yg() As String = Split(Variables.GemHash_Name.Item(DefYellowGem.Text), ";")
            Variables.StoredDefYellowGem = yg(0)
            Dim mg() As String = Split(Variables.GemHash_Name.Item(DefMetaGem.Text), ";")
            Variables.StoredDefMetaGem = mg(0)
            ' Spawn Comparison window.
            Dim F2 As New ItemComparison
            F2.ShowDialog(Me)
        End If
    End Sub

    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutToolStripMenuItem.Click
        Dim F2 As New AboutBox1
        F2.ShowDialog(Me)
    End Sub
    ' END TOOLSTRIP FUNCTIONS
    ''''''''''''''''''''''''''

    ''''''''''''''''''''''''''''
    ' START GEM/ENCHANT BUTTONS
    Private Sub GemButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
    Handles HeadGemButt.Click, NeckGemButt.Click, ShoulderGemButt.Click, BackGemButt.Click, ChestGemButt.Click, WristGemButt.Click, _
    MainHandGemButt.Click, OffhandGemButt.Click, HandsGemButt.Click, WaistGemButt.Click, Finger1GemButt.Click, Finger2GemButt.Click, _
    Trinket1GemButt.Click, Trinket2GemButt.Click, RelicGemButt.Click, LegsGemButt.Click, FeetGemButt.Click
        Dim whichButton As String = sender.name.ToString
        Dim F2 As New GemSelector
        F2.gemwindow = whichButton
        F2.ShowDialog(Me)
        Me.updateRatings()
    End Sub

    Private Sub EnchantButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
    Handles HeadEnchButt.Click, ShoulderEnchButt.Click, BackEnchButt.Click, ChestEnchButt.Click, WristEnchButt.Click, MainHandEnchButt.Click, _
    OffHandEnchButt.Click, HandsEnchButt.Click, LegsEnchButt.Click, FeetEnchButt.Click, WaistEnchButt.Click
        Dim whichbutton As String = sender.name.ToString
        Dim slot As String = Replace(whichbutton, "EnchButt", "")
        If (Variables.CurrentGear.Item(slot) <> "") Then
            Dim F2 As New EnchantSelector
            F2.enchantWindow = whichbutton
            F2.ShowDialog(Me)
            ' The Belt Buckle enchant adds a gem slot.
            If (Variables.CurrentEnchants.Item("Waist") = "1230012") Then
                WaistGemButt.Show()
            Else
                ' Make sure we don't hide the button if they remove the enchant when there is still a socket left.
                Dim hi() As String = Split(Variables.CurrentGear.Item("Waist"), ";")
                If (hi.Count > 1) Then
                    If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then WaistGemButt.Show() Else WaistGemButt.Hide()
                End If
            End If
        Else
                MessageBox.Show("No item selected.", "No item", MessageBoxButtons.OK, MessageBoxIcon.Hand)
        End If
        Me.updateRatings()
    End Sub
    ' END GEM/ENCHANT BUTTONS
    ''''''''''''''''''''''''''

    ''''''''''''''''''''''''''''''
    ' START DROPDOWN INDEX CHANGE
    Private Sub Gear_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
    Handles HeadCombo.SelectedIndexChanged, NeckCombo.SelectedIndexChanged, ShouldersCombo.SelectedIndexChanged, BackCombo.SelectedIndexChanged, _
    ChestCombo.SelectedIndexChanged, WristCombo.SelectedIndexChanged, MainHandCombo.SelectedIndexChanged, OffHandCombo.SelectedIndexChanged, _
    HandsCombo.SelectedIndexChanged, WaistCombo.SelectedIndexChanged, Finger1Combo.SelectedIndexChanged, Finger2Combo.SelectedIndexChanged, _
    Trinket1Combo.SelectedIndexChanged, Trinket2Combo.SelectedIndexChanged, RelicCombo.SelectedIndexChanged, LegsCombo.SelectedIndexChanged, _
    FeetCombo.SelectedIndexChanged
        If Variables.LoadDisable = False Then
            Select Case sender.name.ToString
                Case "HeadCombo"
                    If (HeadCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Head") = ""
                        Variables.CurrentGems.Item("Head") = "0;0;0"
                        Variables.CurrentEnchants.Item("Head") = ""
                        HeadGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Head") = Variables.AllItemsArray_Name.Item(HeadCombo.Text)
                        Variables.CurrentGems.Item("Head") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Head"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then HeadGemButt.Show() Else HeadGemButt.Hide()
                    End If
                Case "NeckCombo"
                    If (NeckCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Neck") = ""
                        Variables.CurrentGems.Item("Neck") = "0;0;0"
                        Variables.CurrentEnchants.Item("Neck") = ""
                        NeckGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Neck") = Variables.AllItemsArray_Name.Item(NeckCombo.Text)
                        Variables.CurrentGems.Item("Neck") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Neck"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then NeckGemButt.Show() Else NeckGemButt.Hide()
                    End If
                Case "ShouldersCombo"
                    If (ShouldersCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Shoulder") = ""
                        Variables.CurrentGems.Item("Shoulder") = "0;0;0"
                        Variables.CurrentEnchants.Item("Shoulder") = ""
                        ShoulderGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Shoulder") = Variables.AllItemsArray_Name.Item(ShouldersCombo.Text)
                        Variables.CurrentGems.Item("Shoulder") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Shoulder"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then ShoulderGemButt.Show() Else ShoulderGemButt.Hide()
                    End If
                Case "BackCombo"
                    If (BackCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Back") = ""
                        Variables.CurrentGems.Item("Back") = "0;0;0"
                        Variables.CurrentEnchants.Item("Back") = ""
                        BackGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Back") = Variables.AllItemsArray_Name.Item(BackCombo.Text)
                        Variables.CurrentGems.Item("Back") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Back"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then BackGemButt.Show() Else BackGemButt.Hide()
                    End If
                Case "ChestCombo"
                    If (ChestCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Chest") = ""
                        Variables.CurrentGems.Item("Chest") = "0;0;0"
                        Variables.CurrentEnchants.Item("Chest") = ""
                        ChestGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Chest") = Variables.AllItemsArray_Name.Item(ChestCombo.Text)
                        Variables.CurrentGems.Item("Chest") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Chest"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then ChestGemButt.Show() Else ChestGemButt.Hide()
                    End If
                Case "WristCombo"
                    If (WristCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Wrist") = ""
                        Variables.CurrentGems.Item("Wrist") = "0;0;0"
                        Variables.CurrentEnchants.Item("Wrist") = ""
                        WristGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Wrist") = Variables.AllItemsArray_Name.Item(WristCombo.Text)
                        Variables.CurrentGems.Item("Wrist") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Wrist"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then WristGemButt.Show() Else WristGemButt.Hide()
                    End If
                Case "MainHandCombo"
                    If (MainHandCombo.Text = "None") Then
                        Variables.CurrentGear.Item("MainHand") = ""
                        Variables.CurrentGems.Item("MainHand") = "0;0;0"
                        Variables.CurrentEnchants.Item("MainHand") = ""
                        MainHandGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("MainHand") = Variables.AllItemsArray_Name.Item(MainHandCombo.Text)
                        Variables.CurrentGems.Item("MainHand") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("MainHand"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then MainHandGemButt.Show() Else MainHandGemButt.Hide()
                    End If
                Case "OffHandCombo"
                    If (OffHandCombo.Text = "None") Then
                        Variables.CurrentGear.Item("OffHand") = ""
                        Variables.CurrentGems.Item("OffHand") = "0;0;0"
                        Variables.CurrentEnchants.Item("OffHand") = ""
                        OffhandGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("OffHand") = Variables.AllItemsArray_Name.Item(OffHandCombo.Text)
                        Variables.CurrentGems.Item("OffHand") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("OffHand"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then OffhandGemButt.Show() Else OffhandGemButt.Hide()
                    End If
                Case "HandsCombo"
                    If (HandsCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Hands") = ""
                        Variables.CurrentGems.Item("Hands") = "0;0;0"
                        Variables.CurrentEnchants.Item("Hands") = ""
                        HandsGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Hands") = Variables.AllItemsArray_Name.Item(HandsCombo.Text)
                        Variables.CurrentGems.Item("Hands") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Hands"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then HandsGemButt.Show() Else HandsGemButt.Hide()
                    End If
                Case "WaistCombo"
                    If (WaistCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Waist") = ""
                        Variables.CurrentGems.Item("Waist") = "0;0;0"
                        Variables.CurrentEnchants.Item("Waist") = ""
                        WaistGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Waist") = Variables.AllItemsArray_Name.Item(WaistCombo.Text)
                        Variables.CurrentGems.Item("Waist") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Waist"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then WaistGemButt.Show() Else WaistGemButt.Hide()
                        If Variables.CurrentEnchants.Item("Waist") = "1230012" Then WaistGemButt.Show()
                    End If
                Case "Finger1Combo"
                    If (Finger1Combo.Text = "None") Then
                        Variables.CurrentGear.Item("Finger1") = ""
                        Variables.CurrentGems.Item("Finger1") = "0;0;0"
                        Variables.CurrentEnchants.Item("Finger1") = ""
                        Finger1GemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Finger1") = Variables.AllItemsArray_Name.Item(Finger1Combo.Text)
                        Variables.CurrentGems.Item("Finger1") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Finger1"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then Finger1GemButt.Show() Else Finger1GemButt.Hide()
                    End If
                Case "Finger2Combo"
                    If (Finger2Combo.Text = "None") Then
                        Variables.CurrentGear.Item("Finger2") = ""
                        Variables.CurrentGems.Item("Finger2") = "0;0;0"
                        Variables.CurrentEnchants.Item("Finger2") = ""
                        Finger2GemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Finger2") = Variables.AllItemsArray_Name.Item(Finger2Combo.Text)
                        Variables.CurrentGems.Item("Finger2") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Finger2"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then Finger2GemButt.Show() Else Finger2GemButt.Hide()
                    End If
                Case "Trinket1Combo"
                    If (Trinket1Combo.Text = "None") Then
                        Variables.CurrentGear.Item("Trinket1") = ""
                        Variables.CurrentGems.Item("Trinket1") = "0;0;0"
                        Variables.CurrentEnchants.Item("Trinket1") = ""
                        Trinket1GemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Trinket1") = Variables.AllItemsArray_Name.Item(Trinket1Combo.Text)
                        Variables.CurrentGems.Item("Trinket1") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Trinket1"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then Trinket1GemButt.Show() Else Trinket1GemButt.Hide()
                    End If
                Case "Trinket2Combo"
                    If (Trinket2Combo.Text = "None") Then
                        Variables.CurrentGear.Item("Trinket2") = ""
                        Variables.CurrentGems.Item("Trinket2") = "0;0;0"
                        Variables.CurrentEnchants.Item("Trinket2") = ""
                        Trinket2GemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Trinket2") = Variables.AllItemsArray_Name.Item(Trinket2Combo.Text)
                        Variables.CurrentGems.Item("Trinket2") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Trinket2"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then Trinket2GemButt.Show() Else Trinket2GemButt.Hide()
                    End If
                Case "RelicCombo"
                    If (RelicCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Relic") = ""
                        Variables.CurrentGems.Item("Relic") = "0;0;0"
                        Variables.CurrentEnchants.Item("Relic") = ""
                        RelicGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Relic") = Variables.AllItemsArray_Name.Item(RelicCombo.Text)
                        Variables.CurrentGems.Item("Relic") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Relic"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then RelicGemButt.Show() Else RelicGemButt.Hide()
                    End If
                Case "LegsCombo"
                    If (LegsCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Legs") = ""
                        Variables.CurrentGems.Item("Legs") = "0;0;0"
                        Variables.CurrentEnchants.Item("Legs") = ""
                        LegsGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Legs") = Variables.AllItemsArray_Name.Item(LegsCombo.Text)
                        Variables.CurrentGems.Item("Legs") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Legs"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then LegsGemButt.Show() Else LegsGemButt.Hide()
                    End If
                Case "FeetCombo"
                    If (FeetCombo.Text = "None") Then
                        Variables.CurrentGear.Item("Feet") = ""
                        Variables.CurrentGems.Item("Feet") = "0;0;0"
                        Variables.CurrentEnchants.Item("Feet") = ""
                        FeetGemButt.Hide()
                    Else
                        Variables.CurrentGear.Item("Feet") = Variables.AllItemsArray_Name.Item(FeetCombo.Text)
                        Variables.CurrentGems.Item("Feet") = "0;0;0"
                        Dim hi() As String = Split(Variables.CurrentGear.Item("Feet"), ";")
                        If (hi(5) <> "0" Or hi(6) <> "0" Or hi(7) <> "0") And (hi(5) <> "" Or hi(6) <> "" Or hi(7) <> "") Then FeetGemButt.Show() Else FeetGemButt.Hide()
                    End If
            End Select
            Armory.processGearInfo()
            Me.updateRatings()
            If sanityCheck() Then
                Me.preRunTasks()
                Other.doModelRun()
                Other.parseModelResults()
                Me.updateCompareBoxes()
            End If
        End If
    End Sub
    ' END DROPDOWN INDEX CHANGE
    ''''''''''''''''''''''''''''

    ''''''''''''''''''''
    ' START SET BUTTONS
    Private Sub SetSaveButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SetSaveButton.Click
        If SetCombo.Text <> "" Then
            Dim output As String = ""
            output &= SetCombo.Text & ","
            output &= DataFiles.getItemIDFromName(HeadCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(NeckCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(ShouldersCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(BackCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(ChestCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(WristCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(MainHandCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(OffHandCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(RelicCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(HandsCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(WaistCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(LegsCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(FeetCombo.Text) & ","
            output &= DataFiles.getItemIDFromName(Finger1Combo.Text) & ","
            output &= DataFiles.getItemIDFromName(Finger2Combo.Text) & ","
            output &= DataFiles.getItemIDFromName(Trinket1Combo.Text) & ","
            output &= DataFiles.getItemIDFromName(Trinket2Combo.Text) & "."
            output &= Variables.CurrentGems.Item("Head") & ","
            output &= Variables.CurrentGems.Item("Neck") & ","
            output &= Variables.CurrentGems.Item("Shoulder") & ","
            output &= Variables.CurrentGems.Item("Back") & ","
            output &= Variables.CurrentGems.Item("Chest") & ","
            output &= Variables.CurrentGems.Item("Wrist") & ","
            output &= Variables.CurrentGems.Item("MainHand") & ","
            output &= Variables.CurrentGems.Item("OffHand") & ","
            output &= Variables.CurrentGems.Item("Relic") & ","
            output &= Variables.CurrentGems.Item("Hands") & ","
            output &= Variables.CurrentGems.Item("Waist") & ","
            output &= Variables.CurrentGems.Item("Legs") & ","
            output &= Variables.CurrentGems.Item("Feet") & ","
            output &= Variables.CurrentGems.Item("Finger1") & ","
            output &= Variables.CurrentGems.Item("Finger2") & ","
            output &= Variables.CurrentGems.Item("Trinket1") & ","
            output &= Variables.CurrentGems.Item("Trinket2") & "."
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Head"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Neck"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Shoulder"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Back"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Chest"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Wrist"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("MainHand"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("OffHand"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Relic"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Hands"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Waist"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Legs"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Feet"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Finger1"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Finger2"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Trinket1"), 0) & ","
            output &= DataFiles.getEnchantInfo(Variables.CurrentEnchants.Item("Trinket2"), 0)
            Variables.GearSetHash.Item(SetCombo.Text) = output
            If Not SetCombo.Items.Contains(SetCombo.Text) Then SetCombo.Items.Add(SetCombo.Text)
            MessageBox.Show("Set saved")
        End If
    End Sub

    Private Sub SetLoadButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SetLoadButton.Click
        If SetCombo.Text <> "" Then
            Variables.LoadDisable = True
            Variables.CurrentGems.Clear()
            Variables.CurrentGear.Clear()
            Variables.CurrentEnchants.Clear()
            Dim r3() As String = Split(Variables.GearSetHash.Item(SetCombo.Text), ".") ' Gear Set 1
            Dim r30() As String = Split(r3(0), ",") ' Items
            Dim r31() As String = Split(r3(1), ",") ' Gems
            Dim r32() As String = Split(r3(2), ",") ' Enchants
            HeadCombo.Text = DataFiles.getItemNameFromId(r30(1))
            getItemfromTextbox("Head", HeadCombo.Text)
            NeckCombo.Text = DataFiles.getItemNameFromId(r30(2))
            getItemfromTextbox("Neck", NeckCombo.Text)
            ShouldersCombo.Text = DataFiles.getItemNameFromId(r30(3))
            getItemfromTextbox("Shoulder", ShouldersCombo.Text)
            BackCombo.Text = DataFiles.getItemNameFromId(r30(4))
            getItemfromTextbox("Back", BackCombo.Text)
            ChestCombo.Text = DataFiles.getItemNameFromId(r30(5))
            getItemfromTextbox("Chest", ChestCombo.Text)
            WristCombo.Text = DataFiles.getItemNameFromId(r30(6))
            getItemfromTextbox("Wrist", WristCombo.Text)
            MainHandCombo.Text = DataFiles.getItemNameFromId(r30(7))
            getItemfromTextbox("MainHand", MainHandCombo.Text)
            OffHandCombo.Text = DataFiles.getItemNameFromId(r30(8))
            getItemfromTextbox("OffHand", OffHandCombo.Text)
            RelicCombo.Text = DataFiles.getItemNameFromId(r30(9))
            getItemfromTextbox("Relic", RelicCombo.Text)
            HandsCombo.Text = DataFiles.getItemNameFromId(r30(10))
            getItemfromTextbox("Hands", HandsCombo.Text)
            WaistCombo.Text = DataFiles.getItemNameFromId(r30(11))
            getItemfromTextbox("Waist", WaistCombo.Text)
            LegsCombo.Text = DataFiles.getItemNameFromId(r30(12))
            getItemfromTextbox("Legs", LegsCombo.Text)
            FeetCombo.Text = DataFiles.getItemNameFromId(r30(13))
            getItemfromTextbox("Feet", FeetCombo.Text)
            Finger1Combo.Text = DataFiles.getItemNameFromId(r30(14))
            getItemfromTextbox("Finger1", Finger1Combo.Text)
            Finger2Combo.Text = DataFiles.getItemNameFromId(r30(15))
            getItemfromTextbox("Finger2", Finger2Combo.Text)
            Trinket1Combo.Text = DataFiles.getItemNameFromId(r30(16))
            getItemfromTextbox("Trinket1", Trinket1Combo.Text)
            Trinket2Combo.Text = DataFiles.getItemNameFromId(r30(17))
            getItemfromTextbox("Trinket2", Trinket2Combo.Text)
            ' Gems
            Variables.CurrentGems.Add("Head", r31(0))
            Variables.CurrentGems.Add("Neck", r31(1))
            Variables.CurrentGems.Add("Shoulder", r31(2))
            Variables.CurrentGems.Add("Back", r31(3))
            Variables.CurrentGems.Add("Chest", r31(4))
            Variables.CurrentGems.Add("Wrist", r31(5))
            Variables.CurrentGems.Add("MainHand", r31(6))
            Variables.CurrentGems.Add("OffHand", r31(7))
            Variables.CurrentGems.Add("Relic", r31(8))
            Variables.CurrentGems.Add("Hands", r31(9))
            Variables.CurrentGems.Add("Waist", r31(10))
            Variables.CurrentGems.Add("Legs", r31(11))
            Variables.CurrentGems.Add("Feet", r31(12))
            Variables.CurrentGems.Add("Finger1", r31(13))
            Variables.CurrentGems.Add("Finger2", r31(14))
            Variables.CurrentGems.Add("Trinket1", r31(15))
            Variables.CurrentGems.Add("Trinket2", r31(16))
            ' Enchants
            Variables.CurrentEnchants.Add("Head", getEnchantInfo(r32(0), 1))
            Variables.CurrentEnchants.Add("Neck", getEnchantInfo(r32(1), 1))
            Variables.CurrentEnchants.Add("Shoulder", getEnchantInfo(r32(2), 1))
            Variables.CurrentEnchants.Add("Back", getEnchantInfo(r32(3), 1))
            Variables.CurrentEnchants.Add("Chest", getEnchantInfo(r32(4), 1))
            Variables.CurrentEnchants.Add("Wrist", getEnchantInfo(r32(5), 1))
            Variables.CurrentEnchants.Add("MainHand", getEnchantInfo(r32(6), 1))
            Variables.CurrentEnchants.Add("OffHand", getEnchantInfo(r32(7), 1))
            Variables.CurrentEnchants.Add("Relic", getEnchantInfo(r32(8), 1))
            Variables.CurrentEnchants.Add("Hands", getEnchantInfo(r32(9), 1))
            Variables.CurrentEnchants.Add("Waist", getEnchantInfo(r32(10), 1))
            Variables.CurrentEnchants.Add("Legs", getEnchantInfo(r32(11), 1))
            Variables.CurrentEnchants.Add("Feet", getEnchantInfo(r32(12), 1))
            Variables.CurrentEnchants.Add("Finger1", getEnchantInfo(r32(13), 1))
            Variables.CurrentEnchants.Add("Finger2", getEnchantInfo(r32(14), 1))
            Variables.CurrentEnchants.Add("Trinket1", getEnchantInfo(r32(15), 1))
            Variables.CurrentEnchants.Add("Trinket2", getEnchantInfo(r32(16), 1))
            Variables.LoadDisable = False

            Armory.processGearInfo()
            Other.doTalentCheck()
            Me.updateRatings()
            Me.preRunTasks()
            Other.doModelRun()
            Other.parseModelResults()
            Me.updateCompareBoxes()
            Me.updateGemBoxes()
        End If
    End Sub

    Private Sub SetDeleteButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SetDeleteButton.Click
        If SetCombo.Text <> "" Then
            Variables.GearSetHash.Remove(SetCombo.Text)
            SetCombo.Items.Remove(SetCombo.Text)
            MessageBox.Show("Set deleted")
        End If
    End Sub
    ' END SET BUTTONS
    ''''''''''''''''''

    Private Function checkValues()
        Dim result As Boolean = True
        If SpellPower.Text = "" Or CritPercent.Text = "" Then
            MessageBox.Show("Enter some stats.", "Invalid Everything", MessageBoxButtons.OK, MessageBoxIcon.Hand)
            result = False
        End If
        Return result
    End Function

    Private Function sanityCheck() As Boolean
        ' Very basic sanity check to see if the model or sim can be executed. (Done with no user interaction when they change gear, etc)
        Dim result As Boolean = True
        If (FightLength.Text = "") Then result = False
        If (MobLevel.Text = "") Then result = False
        If (RaceSelect.Text = "") Then result = False

        Dim spell As Boolean = False
        If (Spells_LB.Checked) Then
            spell = True
        ElseIf (Spells_CL.Checked) Then
            spell = True
        ElseIf (Spells_FS.Checked) Then
            spell = True
        ElseIf (Spells_LvB.Checked) Then
            spell = True
        ElseIf (Spells_UE.Checked) Then
            spell = True
        End If
        If (spell = False) Then result = False
        Return result
    End Function

    Public Sub preRunTasks()
        ' Must be called AFTER updateRatings() since moveToStorage* functions read Textboxes that updateRatings() alters.
        ' Blank the damage array.
        Variables.rawDamage.Clear()
        ' Store values in global.
        Other.moveToStorageChar()
        Other.moveToStorageStats()
        ' Determine bonuses from Talents and update global variables.
        Other.doTalentCheck()
    End Sub
End Class
